/***************************************************************************
Name: JSView
Description: View the source code of external js and css files
Author: Ron Beckman

Copyright (C) 2008  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/




window.addEventListener("load", jsviewLoad, false);

function jsviewLoad() {
	jsview.init();
}

window.addEventListener("unload", jsviewUnload, false);

function jsviewUnload() {
	jsview.exit();
}

var jsviewProgress = {

	_WPL: Components.interfaces.nsIWebProgressListener,

	getObserver: function() {
		return Components.classes["@mozilla.org/observer-service;1"].
		getService(Components.interfaces.nsIObserverService);
	},

	QueryInterface: function(aIID) {
		if (aIID.equals(Components.interfaces.nsIWebProgressListener)   ||
		aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
		aIID.equals(Components.interfaces.nsISupports))
			return this;
		throw Components.results.NS_NOINTERFACE;
	},

	onLinkIconAvailable: function(a) { },

	onLocationChange: function(aWebProgress, aRequest, aLocation) {
		var loading = false;
		var loc = (aLocation) ? aLocation.spec : null;

		if (aWebProgress) {
			try {
				aWebProgress.QueryInterface(Components.interfaces.nsIWebProgress);
				loading = aWebProgress.isLoadingDocument;
			} catch(e) { }
		}

		if (aRequest) {
			try {
				aRequest.QueryInterface(Components.interfaces.nsIRequest);
				loading = aRequest.isPending();
			} catch(e) { }
		}

		if (!loading || !loc || loc == 'about:blank') {
			jsview.load();
		}
	},

	onProgressChange: function(a,b,c,d,e,f) { },
	onSecurityChange: function(a,b,c) { },

	onStateChange: function(aWebProgress, aRequest, aStateFlags, aStatus) {
		if (aStateFlags & this._WPL.STATE_STOP) {
			if (aStateFlags & this._WPL.STATE_IS_NETWORK) {
				if (aWebProgress.DOMWindow == content) {
					jsview.load();
				}
			}
		}
	},

	onStatusChange: function(a,b,c,d) { }
};

var jsviewPrefObserver = {

	prefs: null,

	init: function() {
		this.prefs = this.getPrefs();
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
		this.prefs.addObserver("", this, false);
	},

	uninit: function() {
		this.prefs.removeObserver("", this);
	},

	getPrefs: function() {
		return Components.classes["@mozilla.org/preferences-service;1"].
		getService(Components.interfaces.nsIPrefService).
		getBranch("jsview.");
	},

	observe: function(subject, topic, data) {
		if (topic != "nsPref:changed") return false;

		switch(data) {
			case "hideContext":
			   jsview.hideContext();
			   break;

			case "hideStatus":
			   jsview.hideStatus();
			   break;

			case "hideView":
			   jsview.hideView();
			   break;
		}
		return false;
	}
};

var jsview = {

	getDOM: function() {
		var viw = document.getElementById('jsview-view');
		var btn = document.getElementById('jsview-button');
		var sts = document.getElementById('jsview-status');
		var ctx = document.getElementById('jsview-context');
		var MVPFF = document.getElementById('menu_viewPopup');
		var MVPSM = document.getElementById('menu_View_Popup');
		var CACM = document.getElementById('contentAreaContextMenu');

		return {viw:viw,btn:btn,sts:sts,ctx:ctx,MVPFF:MVPFF,MVPSM:MVPSM,CACM:CACM};
	},

	init: function() {
		//add progress listener for all location changes and page loads
		getBrowser().addProgressListener(jsviewProgress);

		//add observer to monitor jsview prefs
		jsviewPrefObserver.init();

		//show-hide statusbar icon
		this.hideStatus();

		//show-hide context menu item
		this.hideContext();

		//show-hide view menu item
		this.hideView();

		//add onclick event to VPS menuitem on context menu so
		//users can right click and access jsview behavior win.
		//we have to do this dynamically instead of in the
		//overlay since Songbird does not include a VPS menuitem
		//on the context menu by default
		var vps = document.getElementById('context-viewsource');
		if (vps) {
			var oncmd = vps.getAttribute('oncommand');
			if (oncmd) {
				if (oncmd.substr(-1) != ';') oncmd += ';';
				vps.removeAttribute('oncommand');
				vps.setAttribute('onclick','if (event.which == 1) '+oncmd+' else jsvutils.promptBehavior(event,this.parentNode,window.content.location.href);');
				vps.setAttribute('class','menuitem-iconic');
			}
		}

		//add onclick event to VPS menuitem on view menu so users
		//can right click and access jsview behavior window
		var dom = this.getDOM();
		var brows = jsvutils.getAppInfo();
		var MVP = (brows.name == 'SeaMonkey') ? dom.MVPSM : dom.MVPFF;
		for (var i=0; i<MVP.childNodes.length; ++i) {
			if (MVP.childNodes[i].nodeName == 'menuitem' &&
			    MVP.childNodes[i].getAttribute('command') == 'View:PageSource') {
				MVP.childNodes[i].setAttribute('onclick','jsvutils.promptBehavior(event,this.parentNode,window.content.location.href);');
				MVP.childNodes[i].setAttribute('class','menuitem-iconic');
				MVP.childNodes[i].setAttribute('id','viewmenu-viewsource');
				break;
			}
		}

		//if about:blank is set as homepage then icons are not
		//set on load so we have to call the load method here
		if (window.content.location.href == 'about:blank') this.load();
	},

	exit: function() {
		//remove progress listener for location changes and page loads
		getBrowser().removeProgressListener(jsviewProgress);

		//remove pref observer
		jsviewPrefObserver.uninit();

		//remove temp files
		var file,uri;
		var prefs = jsvutils.getPrefs();
		if (prefs.getBoolPref('clearTemp')) {
			var tempDir = jsvutils.getTempDirectory();
			var files = tempDir.directoryEntries;
			while (files.hasMoreElements()) {
				file = files.getNext();
				file.QueryInterface(Components.interfaces.nsIFile);
				if (file.isFile()) {
					uri = jsvutils.convertFileToURI(file);
					uri.QueryInterface(Components.interfaces.nsIURL);
					if (uri.fileExtension == 'jsview' ||
					    uri.fileBaseName.substr(0,7) == 'jsview.') {
						file.remove(false);
					}
				}
			}
		}
	},

	//this method controls the jsview statusbar and toolbar icons
	load: function() {
		var dom = this.getDOM();
		var prefs = jsvutils.getPrefs();
		var status = prefs.getBoolPref('hideStatus');
		if (!dom.btn && status) return false;

		var icon,hidden;
		var js = jsvutils.getExternalJS(window.content,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(window.content,false,true,true,true,true,true,true);

		if (js.length > 0 && css.length > 0) {
			icon = 'both';
			hidden = 'false';
		} else if (js.length > 0) {
			icon = 'js';
			hidden = 'false';
		} else if (css.length > 0) {
			icon = 'css';
			hidden = 'false';
		} else {
			icon = 'none';
			hidden = 'true';
		}

		var label = jsvutils.getStr('jsview.JS') +
				' (' + js.length + ') ' +
				jsvutils.getStr('jsview.CSS') +
				' (' + css.length + ')';

		if (!status && dom.sts) {
			dom.sts.setAttribute('jsview',icon);
			dom.sts.setAttribute('tooltiptext',label);
		}
		if (dom.btn) {
			dom.btn.setAttribute('jsview',icon);
			dom.btn.setAttribute('tooltiptext',label);
		}
		return false;
	},

	populateMenu: function(menu) {
		while (menu.firstChild) {
			menu.removeChild(menu.firstChild);
		}

		var uri,attrs,fname,cname,size,entry,cacheKey;
		var prefs = jsvutils.getPrefs();
		var js = jsvutils.getExternalJS(window.content,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(window.content,false,true,true,true,true,true,true);
		var cache = jsvutils.getCacheService();
		var filesize = prefs.getBoolPref('fileSizeInKB');

		//get string translations
		var done = jsvutils.getStr('jsview.Done');
		var jsnf = jsvutils.getStr('jsview.JsNF');
		var cssnf = jsvutils.getStr('jsview.CssNF');
		var noname = jsvutils.getStr('jsview.NoName');
		var kb = jsvutils.getStr('jsview.KB');
		var bytes = jsvutils.getStr('jsview.Bytes');
		var jsall = jsvutils.getStr('jsview.ViewAllJs');
		var cssall = jsvutils.getStr('jsview.ViewAllCss');

		if (js.length < 1) {
			attrs = ['menuitem',
				  ['label',jsnf],
				  ['tooltiptext',jsnf],
				  ['disabled','true'],
				  ['class','menuitem-iconic jsview-none']];
			menu.appendChild(this.makeElement(attrs));
		} else {
			attrs = ['menuitem',
				  ['label',jsall],
				  ['tooltiptext',jsall],
				  ['oncommand',"jsvutils.viewAll('js',window.content);"],
				  ['class','menuitem-iconic jsview-js jsview-boldNormal']];
			menu.appendChild(this.makeElement(attrs));
		}

		for (var i=0; i<js.length; ++i) {
			//get filename from url
			fname = (js[i].fileName) ? js[i].fileName : noname;

			//add italic for chrome urls
			cname = (js[i].scheme.search(/^chrome$|^resource$/) > -1) ? 'jsview-js jsview-italic' : 'jsview-js';

			//get filesize for url if cached
			cacheKey = js[i].spec.replace(/#.*$/,'');
			try {
				entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
				if (filesize) {
					size = ' (' + (entry.dataSize / 1024).toFixed(2) + ' ' + kb + ')';
				} else {
					size = ' (' + jsvutils.addCommas(entry.dataSize) + ' ' + bytes + ')';
				}
			} catch(e) {
				size = '';
			}

			attrs = ['menuitem',
				  ['label',fname + size],
				  ['value',js[i].spec],
				  ['statustext',js[i].spec],
				  ['tooltiptext',js[i].spec],
				  ['oncommand','jsvutils.viewSource(this.value,4);'],
				  ['onclick','jsvutils.promptBehavior(event,this.parentNode,this.value);'],
				  ['onmouseover','window.content.status = this.value;'],
				  ['onmouseout',"window.content.status = '" + done + "';"],
				  ['class','menuitem-iconic ' + cname]];
			menu.appendChild(this.makeElement(attrs));
		}

		menu.appendChild(this.makeElement(['menuseparator']));

		if (css.length < 1) {
			attrs = ['menuitem',
				  ['label',cssnf],
				  ['tooltiptext',cssnf],
				  ['disabled','true'],
				  ['class','menuitem-iconic jsview-none']];
			menu.appendChild(this.makeElement(attrs));
		} else {
			attrs = ['menuitem',
				  ['label',cssall],
				  ['tooltiptext',cssall],
				  ['oncommand',"jsvutils.viewAll('css',window.content);"],
				  ['class','menuitem-iconic jsview-css jsview-boldNormal']];
			menu.appendChild(this.makeElement(attrs));
		}

		for (var i=0; i<css.length; ++i) {
			//get filename from url
			fname = (css[i].fileName) ? css[i].fileName : noname;

			//add italic for chrome urls
			cname = (css[i].scheme.search(/^chrome$|^resource$/) > -1) ? 'jsview-css jsview-italic' : 'jsview-css';

			//get filesize for url if cached
			cacheKey = css[i].spec.replace(/#.*$/,'');
			try {
				entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
				if (filesize) {
					size = ' (' + (entry.dataSize / 1024).toFixed(2) + ' ' + kb + ')';
				} else {
					size = ' (' + jsvutils.addCommas(entry.dataSize) + ' ' + bytes + ')';
				}
			} catch(e) {
				size = '';
			}

			attrs = ['menuitem',
				  ['label',fname + size],
				  ['value',css[i].spec],
				  ['statustext',css[i].spec],
				  ['tooltiptext',css[i].spec],
				  ['oncommand','jsvutils.viewSource(this.value,4);'],
				  ['onclick','jsvutils.promptBehavior(event,this.parentNode,this.value);'],
				  ['onmouseover','window.content.status = this.value;'],
				  ['onmouseout',"window.content.status = '" + done + "';"],
				  ['class','menuitem-iconic ' + cname]];
			menu.appendChild(this.makeElement(attrs));
		}
	},

	makeElement: function(attrs) {
		var elem = document.createElement(attrs[0]);
		for (var i=1; i<attrs.length; ++i) {
			elem.setAttribute(attrs[i][0],attrs[i][1]);
		}
		return elem;
	},

	hideStatus: function() {
		var dom = this.getDOM();
		if (!dom.sts) return false;

		var prefs = jsvutils.getPrefs();
		dom.sts.setAttribute('hidden',prefs.getBoolPref('hideStatus'));
		return false;
	},

	hideView: function() {
		var dom = this.getDOM();
		if (!dom.viw) return false;

		var brows = jsvutils.getAppInfo();
		var MVP = (brows.name == 'SeaMonkey') ? dom.MVPSM : dom.MVPFF;
		if (!MVP) return false;

		var prefs = jsvutils.getPrefs();
		var hv = prefs.getBoolPref('hideView');
		dom.viw.setAttribute('hidden',hv);

		if (hv) {
			MVP.removeEventListener('popupshowing', jsview.showViewPopup, false);
		} else {
			MVP.addEventListener('popupshowing', jsview.showViewPopup, false);
		}
		return false;
	},

	showViewPopup: function() {
		var dom = jsview.getDOM();
		if (!dom.viw) return false;

		var icon,hidden;
		var js = jsvutils.getExternalJS(window.content,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(window.content,false,true,true,true,true,true,true);

		if (js.length > 0 && css.length > 0) {
			icon = 'both';
			hidden = 'false';
		} else if (js.length > 0) {
			icon = 'js';
			hidden = 'false';
		} else if (css.length > 0) {
			icon = 'css';
			hidden = 'false';
		} else {
			icon = 'none';
			hidden = 'true';
		}

		var label = jsvutils.getStr('jsview.JS') +
				' (' + js.length + ') ' +
				jsvutils.getStr('jsview.CSS') +
				' (' + css.length + ')';

		dom.viw.setAttribute('jsview',icon);
		dom.viw.setAttribute('hidden',hidden);
		dom.viw.setAttribute('tooltiptext',label);
		dom.viw.setAttribute('label',label);
		return false;
	},

	hideContext: function() {
		var dom = this.getDOM();
		if (!dom.ctx || !dom.CACM) return false;

		var prefs = jsvutils.getPrefs();
		var hc = prefs.getBoolPref('hideContext');
		dom.ctx.setAttribute('hidden',hc);

		if (hc) {
			dom.CACM.removeEventListener('popupshowing', jsview.showContextPopup, false);
		} else {
			dom.CACM.addEventListener('popupshowing', jsview.showContextPopup, false);
		}

		return false;
	},

	showContextPopup: function() {
		var dom = jsview.getDOM();
		if (!dom.ctx) return false;

		if (!('gContextMenu' in window) || !gContextMenu) {
			dom.ctx.setAttribute('hidden',true);
			return false;
		}

		var gCM = gContextMenu.inDirList ||
				gContextMenu.onImage ||
				gContextMenu.isContentSelected ||
				gContextMenu.onLink ||
				gContextMenu.onTextInput;

		var icon,hidden;
		var js = jsvutils.getExternalJS(window.content,false,true,true,true,true,true,true);
		var css = jsvutils.getExternalCSS(window.content,false,true,true,true,true,true,true);

		if (js.length > 0 && css.length > 0) {
			icon = 'both';
			hidden = 'false';
		} else if (js.length > 0) {
			icon = 'js';
			hidden = 'false';
		} else if (css.length > 0) {
			icon = 'css';
			hidden = 'false';
		} else {
			icon = 'none';
			hidden = 'true';
		}

		var label = jsvutils.getStr('jsview.JS') +
				' (' + js.length + ') ' +
				jsvutils.getStr('jsview.CSS') +
				' (' + css.length + ')';

		dom.ctx.setAttribute('jsview',icon);
		dom.ctx.setAttribute('hidden',(gCM) ? true : hidden);
		dom.ctx.setAttribute('tooltiptext',label);
		dom.ctx.setAttribute('label',label);

		return false;
	},

	cancelContextMenu: function() {
		var ctxt = document.getElementById('jsview-menupopup');
		ctxt.hidePopup();
	},

	viewAbout: function() {
		//check for existence of window
		var win = jsvutils.getWin('jsview:about');
		if (win) {
			//if exists then bring to front
			win.focus();
		} else {
			//if doesn't exist then open a new window
			window.openDialog("chrome://jsview/content/jsviewAbout.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
		}
	},

	viewOptions: function() {
		//check for existence of window
		var win = jsvutils.getWin('jsview:options');
		if (win) {
			//if exists then bring to front
			win.focus();
		} else {
			//if doesn't exist then open a new window
			window.openDialog("chrome://jsview/content/jsviewOptions.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
		}
	},

	viewObjRef: function() {
		//check for existence of window
		var win = jsvutils.getWin('jsview:objref');
		if (win) {
			//if exists then bring to front
			win.focus();
		} else {
			//if doesn't exist then open a new window
			window.openDialog("chrome://jsview/content/jsviewObjRef.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
		}
	},

	viewPageInfo: function() {
		//check for existence of window
		var win = jsvutils.getWin('jsview:pageinfo');
		if (win) {
			//if exists then bring to front
			win.focus();
		} else {
			//if doesn't exist then open a new window
			window.openDialog("chrome://jsview/content/jsviewPageInfo.xul", "_blank", "chrome,centerscreen,resizable=yes", null);
		}
	}
};
