/***************************************************************************
Name: JSView
Description: View the source code of external js and css files
Author: Ron Beckman

Copyright (C) 2008  Ron Beckman

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

Free Software Foundation, Inc.
51 Franklin Street
Fifth Floor
Boston, MA  02110-1301
USA
***************************************************************************/


window.addEventListener("load", jsviewPageInfoLoad, false);

function jsviewPageInfoLoad() {
	jsviewPageInfo.init();
}

var jsviewPageInfo = {

	_styleElements: [],
	_scriptElements: [],

	getCellData: function(tree,row,col) {
		if (row == null) row = tree.currentIndex;
		if (col == null) col = 0;

		try {
			var cell = tree.view.getCellText(row,tree.columns.getColumnAt(col));
		} catch(e) { }

		return cell;
	},

	init: function() {
		//populate the scripts tree
		this.loadScripts();

		//populate the sheets tree
		this.loadSheets();

		//populate the frames tree
		this.loadFrames();
	},

	loadFrames: function() {
		var url,size,cacheKey,entry;
		var prefs = jsvutils.getPrefs();
		var cache = jsvutils.getCacheService();
		var filesize = prefs.getBoolPref('fileSizeInKB');

		//get string translations
		var kb = jsvutils.getStr('jsview.KB');
		var bytes = jsvutils.getStr('jsview.Bytes');

		//remove all tree items
		var child = document.getElementById('jsview-frChild');
		while (child.firstChild) {
			child.removeChild(child.firstChild);
		}

		var brows = jsvutils.getAppInfo();
		var win = jsvutils.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');

		var frames = jsvutils.getFrames(win.content);
		//we start at 1 since 0 is window.content
		for (var i=1; i<frames.length; ++i) {
			url = frames[i].location.href;
			try {
				//get filesize for url if cached
				cacheKey = url.replace(/#.*$/,'');
				entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
				if (filesize) {
					size = (entry.dataSize / 1024).toFixed(2) + ' ' + kb;
				} else {
					size = jsvutils.addCommas(entry.dataSize) + ' ' + bytes;
				}
			} catch(e) {
				size = '??'; //display something to indicate size unknown
			}
			child.appendChild(this.createItem([url,size]));
		}
	},

	loadScripts: function() {
		var url,size,cacheKey,entry,item,index;
		var prefs = jsvutils.getPrefs();
		var cache = jsvutils.getCacheService();
		var filesize = prefs.getBoolPref('fileSizeInKB');

		//get string translations
		var kb = jsvutils.getStr('jsview.KB');
		var bytes = jsvutils.getStr('jsview.Bytes');
		var unk = jsvutils.getStr('jsview.Unknown');
		var embd = jsvutils.getStr('jsview.Embedded');

		//remove all tree items
		var child = document.getElementById('jsview-jsChild');
		while (child.firstChild) {
			child.removeChild(child.firstChild);
		}

		var brows = jsvutils.getAppInfo();
		var win = jsvutils.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');

		var scripts = jsvutils.getExternalJS(win.content,true,false,false,false,false,false,false);
		for (var i=0; i<scripts.length; ++i) {
			if (scripts[i] instanceof Components.interfaces.nsIDOMHTMLScriptElement) {
				url = embd;
				this._scriptElements[i] = scripts[i];
				if (filesize) {
					size = (scripts[i].innerHTML.length / 1024).toFixed(2) + ' ' + kb;
				} else {
					size = jsvutils.addCommas(scripts[i].innerHTML.length) + ' ' + bytes;
				}
			} else {
				try {
					url = scripts[i];
					cacheKey = url.replace(/#.*$/,'');
					//get filesize for url if cached
					entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
					if (filesize) {
						size = (entry.dataSize / 1024).toFixed(2) + ' ' + kb;
					} else {
						size = jsvutils.addCommas(entry.dataSize) + ' ' + bytes;
					}
				} catch(e) {
					size = unk;
				}
			}

			item = this.createItem([url,size]);
			if (url == embd) item.setAttribute('jsview',i);
			child.appendChild(item);
		}
	},

	loadSheets: function() {
		var url,size,cacheKey,entry,item,index;
		var prefs = jsvutils.getPrefs();
		var cache = jsvutils.getCacheService();
		var filesize = prefs.getBoolPref('fileSizeInKB');

		//get string translations
		var kb = jsvutils.getStr('jsview.KB');
		var bytes = jsvutils.getStr('jsview.Bytes');
		var unk = jsvutils.getStr('jsview.Unknown');
		var embd = jsvutils.getStr('jsview.Embedded');

		//remove all tree items
		var child = document.getElementById('jsview-cssChild');
		while (child.firstChild) {
			child.removeChild(child.firstChild);
		}

		var brows = jsvutils.getAppInfo();
		var win = jsvutils.getWin((brows.name == 'Songbird') ? 'Songbird:Main' : 'navigator:browser');

		var sheets = jsvutils.getExternalCSS(win.content,true,false,false,false,false,true,false);
		for (var i=0; i<sheets.length; ++i) {
			if (sheets[i] instanceof Components.interfaces.nsIDOMHTMLStyleElement) {
				url = embd;
				this._styleElements[i] = sheets[i];
				if (filesize) {
					size = (sheets[i].innerHTML.length / 1024).toFixed(2) + ' ' + kb;
				} else {
					size = jsvutils.addCommas(sheets[i].innerHTML.length) + ' ' + bytes;
				}
			} else {
				try {
					url = sheets[i];
					cacheKey = url.replace(/#.*$/,'');
					//get filesize for url if cached
					entry = cache.openCacheEntry(cacheKey, Components.interfaces.nsICache.ACCESS_READ, false);
					if (filesize) {
						size = (entry.dataSize / 1024).toFixed(2) + ' ' + kb;
					} else {
						size = jsvutils.addCommas(entry.dataSize) + ' ' + bytes;
					}
				} catch(e) {
					size = unk;
				}
			}

			item = this.createItem([url,size]);
			if (url == embd) item.setAttribute('jsview',i);
			child.appendChild(item);
		}
	},

	createItem: function(cells) {
		var cell;
		var item = document.createElement('treeitem');
		var row = document.createElement('treerow');

		for (var i=0; i<cells.length; ++i) {
			cell = document.createElement('treecell');
			cell.setAttribute('label',cells[i]);
			row.appendChild(cell);
		}

		item.appendChild(row);
		return item;
	},

	copyLink: function() {
		var tree = this.getTree();
		if (!tree.view || !tree.view.selection || !tree.columns || !tree.contentView) return false;

		var urls = this.getSelectedUrls(tree);
		jsvutils.copyToClipboard(urls.join('\r\n'));
		return false;
	},

	getTree: function() {
		var id;
		var panel = document.getElementById('tabbox').selectedPanel;
		if (panel.id == 'jsview-jsPanel') {
			id = 'jsview-jsTree';
		} else if (panel.id == 'jsview-cssPanel') {
			id = 'jsview-cssTree';
		} else {
			id = 'jsview-frTree';
		}
		return document.getElementById(id);
	},

	getSelectedUrls: function(tree) {
		var cell;
		var urls = [];
		var start = {};
		var end = {};
		var embd = jsvutils.getStr('jsview.Embedded');
		var range = tree.view.selection.getRangeCount();

		for (var i=0; i<range; ++i) {
			tree.view.selection.getRangeAt(i,start,end);
			for (var j=start.value; j<=end.value; ++j) {
				cell = this.getCellData(tree,j,0)
				if (cell == embd) {
					cell = this.getTempFile(tree,j);
				}
				urls.push(cell);
			}
		}
		return urls;
	},

	getTempFile: function(tree,row) {
		var url;
		var ext = (tree.id == 'jsview-jsTree') ? '.js' : '.css';
		var obj = (tree.id == 'jsview-jsTree') ? '_scriptElements' : '_styleElements';

		var item = tree.contentView.getItemAtIndex(row);
		var index = parseInt(item.getAttribute('jsview'));
		if (this[obj][index].hasAttribute('jsview')) {
			url = this[obj][index].getAttribute('jsview');
		} else {
			var temp = jsvutils.getTempDirectory();
			var dt = new Date();
			var time = dt.getTime();
			var rdm = time.toString().substr(-4);
			var name = 'jsview.' + rdm + ext;
			temp.append(name);
			try {
				this.writeFile(temp, this[obj][index].innerHTML, 0x04 | 0x08 | 0x20);
				url = jsvutils.convertFileToUrl(temp);
			} catch(e) {
				alert(jsvutils.getStr('jsview.WriteError'));
				url = 'about:blank';
			}
			this[obj][index].setAttribute('jsview',url);
		}
		return url;
	},

	viewSource: function(type) {
		var tree = this.getTree();
		if (!tree.view || !tree.view.selection || !tree.columns || !tree.contentView) return false;

		var urls = this.getSelectedUrls(tree);
		for (var i=0; i<urls.length; ++i) {
			setTimeout('jsvutils.viewSource("'+urls[i]+'","'+type+'")',i*500);
		}
		return false;
	},

	writeFile: function(file, fileContent, flags) {
		var strm = jsvutils.getFileOutputStream();
		strm.QueryInterface(Components.interfaces.nsIOutputStream);
		strm.QueryInterface(Components.interfaces.nsISeekableStream);
		strm.init(file, flags, 420, 0);
		strm.write(fileContent, fileContent.length);
		strm.flush();
		strm.close();
	}
};
